@@ -1,19 +1,18 @@
 package Number::Tolerant::Constant;
-use base qw(Number::Tolerant);
-our $VERSION = "1.40";
+our $VERSION = "1.50";
 
 use strict;
 use warnings;
 
 =head1 NAME
 
-Number::Tolerant::Constant -- a blessed constant type
+Number::Tolerant::Constant - a blessed constant type
 
 =head1 VERSION
 
-version 1.40
+version 1.50
 
- $Id: /my/cs/projects/tolerant/trunk/lib/Number/Tolerant/Constant.pm 18115 2006-01-27T01:51:05.875783Z rjbs  $
+ $Id: /my/cs/projects/tolerant/trunk/lib/Number/Tolerant/Constant.pm 18205 2006-01-28T00:48:56.763611Z rjbs  $
 
 =head1 SYNOPSIS
 
@@ -34,14 +33,13 @@ would otherwise complain that the constructor hadn't returned a blessed object.
 
 =cut
 
-my $number = Number::Tolerant->_number_re;
+package Number::Tolerant::Type::constant_obj;
+use base qw(Number::Tolerant::Type);
 
-package Number::Tolerant::Type::constant;
-use base qw(Number::Tolerant);
-no warnings 'redefine';
+my $number = $Number::Tolerant::Type::number;
 
 sub construct { shift;
-	{ value => $_[0], min => $_[0], max => $_[0], constant => 1 }
+  { value => $_[0], min => $_[0], max => $_[0], constant => 1 }
 };
 
 sub parse { shift;
@@ -51,11 +49,13 @@ sub parse { shift;
 sub stringify { $_[0]->{value} }
 
 sub valid_args { shift;
-	return $_[0] if @_==1 and $_[0] =~ m!\A($number)\z!;
-	return;
+  return $_[0] if @_==1 and $_[0] =~ m!\A($number)\z!;
+  return;
 }
 
-Number::Tolerant->_tolerance_type->{'Number::Tolerant::Type::constant'} = 1;
+#Number::Tolerant->_tolerance_type->{'Number::Tolerant::Type::constant'} = 1;
+Number::Tolerant->disable_plugin("Number::Tolerant::Type::constant");
+Number::Tolerant->enable_plugin( "Number::Tolerant::Type::constant_obj");
 
 =head1 TODO
 
@@ -65,8 +65,8 @@ Ricardo SIGNES, E<lt>rjbs@cpan.orgE<gt>
 
 =head1 COPYRIGHT
 
-(C) 2004, Ricardo SIGNES.  Number::Tolerant::Constant is available under the
-same terms as Perl itself.
+(C) 2004-2006, Ricardo SIGNES.  Number::Tolerant::Constant is available under
+the same terms as Perl itself.
 
 =cut
 
